class ExtLogger

  module Common

    ENV_HASH = {
        dev: 'development',
        stg: 'staging',
        pro: 'production',
    }

    def self.included(base)
      # base.send(:include, InstanceMethods)
      base.extend ClassMethods
      # base.class_eval do
      #   scope :active, where(is_active: true)
      # end
    end

    module ClassMethods
      # Validation the Rails object
      def valid_rails
        defined?(::Rails)
      end

      # Get the path of folder or rails project
      def rails_root
        self.valid_rails && ::Rails.root ? ::Rails.root.to_s : ::Dir.pwd
      end
      # Get the env
      def rails_env
        self.valid_rails && ::Rails.env ? ::Rails.env : ENV_HASH[:dev]
      end
    end

    # Get class name
    def get_class_name
      self.class.to_s
    end

    # Get instance name
    def get_instance_name
      self.class.to_s.downcase
    end

    # Set debug
    def set_debug(_bool=true)
      @debug = _bool  # Controller debug content output to console
    end

    def get_debug
      return @debug || false
    end

    # Put content to console by debug=true
    def output_debug(str)
      puts "DEBUG_P: #{str}" if @debug
    end

    # Analysis exception object
    def analysis_exception(e)
      func_name = "[common.#{__method__.to_s}]"
      output_debug "#{func_name} e.class: #{e.class}"
      if !e.class.to_s.include?('Error')
        @logger.error("The object 'e:#{e.to_s}' is not exception. e.class: #{e.class}")
        return ''
      end
      str = e.message + "\n" + e.backtrace.join("\n")
      return str
    end

    # File operate
    # Check the folder and create it when it is not exist.
    def mkdir_more(file_path)
      begin
        require 'FileUtils' if !defined? FileUtils
        FileUtils.mkdir_p(File.dirname(file_path))
      rescue Exception => e
        puts e.message + "\n" + e.backtrace.join("\n")
      end

      # rails_flag = true if path.include?(rails_root)
      # if !::Dir.exist?(_path)
      #   _path_array = _path.split('/').select{|item| !item.nil? && item != ''}
      #   _path = rails_flag ? rails_root : ''
      #
      #   _path_array.each_with_index do |item, idx|
      #     break if idx >= _path_array.length - 1
      #     _path += '/' + item
      #     ::Dir.mkdir(_path) if !::Dir.exist?(_path)
      #   end
      # end
    end

    # String
    # Empty judgement for object
    def is_blank?(obj)
      func_name = "[common.#{__method__.to_s}]"
      # output_debug "#{func_name} obj.class: #{obj.class}"
      return true if obj.nil?
      return (obj.is_a?(String) || obj.is_a?(Array) || obj.is_a?(Hash)) && obj.empty?
    end

    # date & time
    TIME_MS_DAY     = 24 * 60 * 60 * 1000 # Day with millisecond
    TIME_MS_HOUR    = 60 * 60 * 1000      # Hour with millisecond
    TIME_MS_MIDDLE  = 60 * 1000           # Middle with millisecond
    TIME_MS_SECOND  = 1000                # Second with millisecond

    TIME_UNIT_DAY         = 'day '
    TIME_UNIT_HOUR        = ':'
    TIME_UNIT_MIDDLE      = ':'
    TIME_UNIT_SECOND      = ' '
    TIME_UNIT_MILLISECOND = 'ms'

    # Formatting time difference
    def format_sub_time(start_time, end_time)
      use_time = (end_time - start_time).to_i
      time_list = []

      if use_time > TIME_MS_DAY
        remainder_time = use_time % TIME_MS_DAY
        num = (use_time - remainder_time) / TIME_MS_DAY
        time_list << "#{num}#{TIME_UNIT_DAY}"
        use_time = remainder_time
      end

      if use_time > TIME_MS_HOUR
        remainder_time = use_time % TIME_MS_HOUR
        num = (use_time - remainder_time) / TIME_MS_HOUR
        time_list << "#{num}#{TIME_UNIT_HOUR}"
        use_time = remainder_time
      end

      if use_time > TIME_MS_MIDDLE
        remainder_time = use_time % TIME_MS_MIDDLE
        num = (use_time - remainder_time) / TIME_MS_MIDDLE
        time_list << "#{num}#{TIME_UNIT_MIDDLE}"
        use_time = remainder_time
      end

      if use_time > TIME_MS_SECOND
        remainder_time = use_time % TIME_MS_SECOND
        num = (use_time - remainder_time) / TIME_MS_SECOND
        time_list << "#{num}#{TIME_UNIT_SECOND}"
        use_time = remainder_time
      end

      if use_time > 0
        time_list << "#{use_time}#{TIME_UNIT_MILLISECOND}"
      end

      return time_list.join('')
    end
  end
end